
# q-expansion of newform 3680.2.a.n, downloaded from the LMFDB on 11 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3680
weight = 2
poly_data = [-3, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [1, 0], [-1, 0], [1, -1], [-1, 1], [0, 1], [2, -2], [-1, 1], [-1, 0], [1, 4], [-2, -1], [-1, 1], [-1, -2], [-5, 1], [1, 4], [-4, -2], [0, -6], [-1, -5], [-3, 5], [0, -7], [-14, -1], [7, 5], [-4, 0], [-9, 5], [1, 5], [-4, 8], [2, -2], [-9, -1], [-11, -5], [-9, -1], [-3, -8], [4, -5], [2, -2], [-10, 3], [-2, -4], [8, 7], [1, 1], [-3, -2], [14, 4], [6, -8], [6, 5], [-9, -5], [8, 6], [-16, -5], [8, -5], [17, 3], [-2, -6], [-10, 4], [3, 1], [-1, -7], [-4, 11], [16, -1], [0, 14], [-8, 4], [2, -1], [5, -11], [-15, 4], [-2, -14], [-10, 11], [-1, -7], [4, 10], [0, 16], [8, -2], [-14, 11], [-12, 4], [14, -4], [-2, -1], [19, -9], [-6, -8], [-11, 0], [18, -1], [19, 5], [18, 2], [-14, 4], [6, 6], [1, 3], [-22, -6], [-4, 1], [18, -8], [-19, 0], [3, -1], [-11, -1], [-11, -3], [2, 4], [20, 3], [9, -14], [4, 8], [24, 4], [-15, 2], [4, 2], [18, -8], [-6, -18], [-13, 6], [-8, 5], [-10, -1], [-8, 6], [15, -10], [30, -6], [-23, -9], [11, 6], [7, -12], [-1, 7], [12, 0], [6, -8], [-2, -10], [4, 17], [-29, 2], [-12, -16], [24, -10], [-5, -4], [4, 6], [-10, -14], [27, 1], [-22, -14], [33, -3], [3, 13], [24, -2], [3, 2], [18, -3], [-28, 12], [-2, -16], [4, 19], [2, -4], [-1, 22], [-18, 6], [9, -1], [11, 9], [26, 6], [4, -18], [12, 12], [4, -21], [18, 18], [1, -15], [7, -3], [19, -12], [-6, 4], [7, -5], [4, 16], [33, 11], [-24, 4], [-8, 11], [-32, 8], [-15, -12], [-10, 6], [-22, -10], [27, -5], [-2, -12], [-6, -11], [2, 5], [-19, -14], [16, -6], [2, 6], [34, 0], [1, 22], [23, -1], [23, 9], [23, 3], [-9, 6], [37, -5], [-4, -24], [37, -2], [-8, 18], [7, 2], [-18, 0], [27, 17], [8, -18], [-44, 0], [-24, -10], [-25, -21], [14, 17], [-15, -17], [25, -10], [-6, -6], [-52, 0], [-18, 1], [15, -7], [44, -7], [7, 26], [-23, 9], [-14, 22], [-33, -14], [17, -23], [16, 15], [12, 18], [36, -2], [16, 4], [36, -8], [39, 7], [-5, -2], [-24, 8], [-2, -19], [44, 2], [38, 16], [-24, 20], [36, 2], [7, -9], [5, 5], [-26, -2], [6, 26], [-29, 12], [-30, -2], [-44, -2], [22, -21], [5, 9], [-40, 16], [40, 9], [7, -15], [-17, 16], [-7, 0], [-12, -15], [-48, 11], [25, -8], [17, -9], [4, 18], [-26, 2], [19, -19], [-13, 12], [-35, 16], [6, 8], [0, -31], [-17, -12], [-7, 15], [11, -14], [-63, -5], [-9, -10], [27, 18], [-39, 7], [-8, 37], [-18, -4], [0, -8], [-50, -4], [-15, 21], [28, -14], [-50, -8], [-2, -2], [9, -9], [-14, 26], [-15, 23], [-36, -7], [6, -1], [40, -4], [62, 3], [-43, -6], [-33, 14], [4, 14], [34, -7], [-19, 18], [-36, 14], [-44, -10], [-6, 24], [-12, 4], [-12, -6], [-40, -2], [-45, 17], [36, -20], [-20, -13], [-41, -9], [-54, 4], [-36, 17], [12, 33], [-32, -8], [37, 19], [-38, 2], [-19, -19], [-8, -12], [9, 39], [-21, -9], [-5, -7], [-7, 9], [6, -9], [-23, 6], [70, -4], [-23, -3], [27, 3], [-10, 1], [-25, 12], [-12, -40], [-38, -6], [-26, 4], [-26, -14], [3, -6], [24, -18], [11, -13], [12, -22], [12, -36], [-32, 4], [-6, -24], [-51, -1], [38, 10], [14, -18], [-24, -14], [-12, -36], [42, 4], [3, 37], [-28, 5], [12, -14], [-56, 8], [22, 15], [-6, -9], [25, 34], [35, 23], [-15, -9], [-10, 18], [-33, 6], [34, 22], [10, 7], [-36, -8], [8, -16], [32, -2], [-38, 12], [3, -30], [-13, -1], [14, -26], [19, -38], [-15, 29], [-1, 29], [-26, -17], [-68, 5], [-8, -36], [13, 32], [-25, -9], [-9, -17], [-19, 10], [-2, -24], [24, 10], [-26, -14], [2, -40], [5, -6], [75, -1], [52, -20], [-17, 0], [-32, -29], [-72, -13], [21, -8], [-23, -13], [9, 20], [16, 10], [-1, 51], [-43, 0], [-27, -9], [-52, 1], [51, -21], [78, -2], [4, 38], [-32, 18], [-4, 40], [29, 24], [4, 46], [0, -30], [-20, 38], [-32, -23], [-44, 15], [-42, 28], [28, 38], [0, 8], [12, -26], [28, -4], [-2, 0], [-14, 15], [-48, 18], [64, 10], [-59, -14], [45, -4], [-18, 0], [1, -32], [20, -43], [-30, -31], [-11, -35], [18, 28], [-72, -1], [64, -16], [15, -24], [6, -34], [50, -18], [-34, 9], [-36, 29], [-3, 47], [30, 20], [-6, 50], [-16, -17], [6, 36], [69, 15], [-16, -10], [-36, -25], [61, 1], [-63, 8], [-38, 31], [-32, 10], [16, 34], [-8, 0], [-14, -30], [0, -32], [-50, 3], [66, 8], [21, 17], [-62, 0], [51, 7], [55, 17], [2, 18], [-18, -46], [15, -44], [36, -5], [-37, 25], [46, -6], [65, 18], [-48, 15], [-35, 27], [11, -18], [-63, -5], [48, 16], [34, -13], [-42, 27], [-3, -39], [12, -10], [-13, -11], [43, -10], [24, 40], [-56, -22], [-29, 11], [-9, -3], [-55, -1], [-49, -5], [66, -16], [-8, -52], [48, -13], [54, 11], [5, -11], [-10, 12], [-52, 5], [36, 25]]
